/*
	File: blop_c.h
	Author: Shivkumar Chandrasekaran
	Date: 8.25.2003
	Email: shiv@ece.ucsb.edu
	Purpose: C glue between Clean/OCaml and Lapack and Blas
	Package: CleanFloat and CamlFloat
	
	$Id$
*/

typedef struct {double re; double im;} complex;
typedef struct {float re; float im;} scomplex;

double my_dlange (int norm, int m, int n, double *a, int disp, int lda, double *work);
double my_zlange (int norm, int m, int n, complex *a, int disp, int lda, double *work);
float my_slange (int norm, int m, int n, float *a, int disp, int lda, float *work);
float my_clange (int norm, int m, int n, scomplex *a, int disp, int lda, float *work);
int my_cblas_dgemm (int order, int transA, int transB, int m, int n, int k, double alpha,
	double *a, int dispa, int lda, double *b, int dispb, int ldb, double beta,
		    double *c, int dispc, int ldc);
int my_cblas_zgemm (int order, int transA, int transB, int m, int n, int k, complex *alpha,
	complex *a, int dispa, int lda, complex *b, int dispb, int ldb, complex *beta,
		    complex *c, int dispc, int ldc);
int my_cblas_sgemm (int order, int transA, int transB, int m, int n, int k, float alpha,
	float *a, int dispa, int lda, float *b, int dispb, int ldb, float beta,
		    float *c, int dispc, int ldc);
int my_cblas_cgemm (int order, int transA, int transB, int m, int n, int k, scomplex *alpha,
	scomplex *a, int dispa, int lda, scomplex *b, int dispb, int ldb, scomplex *beta,
		    scomplex *c, int dispc, int ldc);
int my_dgeqlf (int m, int n, double *a, int dispa, int lda, double *tau,
	       double *work, int lwork);
int my_dgelqf (int m, int n, double *a, int dispa, int lda, double *tau,
	       double *work, int lwork);
int my_sgeqlf (int m, int n, float *a, int dispa, int lda, float *tau,
	       float *work, int lwork);
int my_sgelqf (int m, int n, float *a, int dispa, int lda, float *tau,
	       float *work, int lwork);
int my_zgeqlf (int m, int n, complex *a, int dispa, int lda, complex *tau,
	       complex *work, int lwork);
int my_zgelqf (int m, int n, complex *a, int dispa, int lda, complex *tau,
	       complex *work, int lwork);
int my_zunmql (int side, int trans, int m, int n, int k, complex *a, int dispa, int lda,
	       complex *tau, complex *c, int dispc, int ldc, complex *work, int lwork);
int my_zunmlq (int side, int trans, int m, int n, int k, complex *a, int dispa, int lda,
	       complex *tau, complex *c, int dispc, int ldc, complex *work, int lwork);
int my_cgeqlf (int m, int n, scomplex *a, int dispa, int lda, scomplex *tau,
	       scomplex *work, int lwork);
int my_cgelqf (int m, int n, scomplex *a, int dispa, int lda, scomplex *tau,
	       scomplex *work, int lwork);
int my_cunmql (int side, int trans, int m, int n, int k, scomplex *a, int dispa, int lda,
	       scomplex *tau, scomplex *c, int dispc, int ldc, scomplex *work, int lwork);
int my_cunmlq (int side, int trans, int m, int n, int k, scomplex *a, int dispa, int lda,
	       scomplex *tau, scomplex *c, int dispc, int ldc, scomplex *work, int lwork);
int my_dormql (int side, int trans, int m, int n, int k,
	      double a[], int dispa, int lda, double tau[], double c[], int dispc,
	       int ldc, double work[], int lwork);
int my_dormlq (int side, int trans, int m, int n, int k,
	      double a[], int dispa, int lda, double tau[], double c[], int dispc,
	       int ldc, double work[], int lwork);
int my_sormql (int side, int trans, int m, int n, int k,
	      float a[], int dispa, int lda, float tau[], float c[], int dispc,
	       int ldc, float work[], int lwork);
int my_sormlq (int side, int trans, int m, int n, int k,
	      float a[], int dispa, int lda, float tau[], float c[], int dispc,
	       int ldc, float work[], int lwork);
int my_dlacpy (int uplo, int m, int n, double a[], int dispa, int lda, double b[],
	       int dispb, int ldb);
int my_zlacpy (int uplo, int m, int n, complex *a, int dispa, int lda, complex *b,
	       int dispb, int ldb);
int my_slacpy (int uplo, int m, int n, float a[], int dispa, int lda, float b[],
	       int dispb, int ldb);
int my_clacpy (int uplo, int m, int n, scomplex *a, int dispa, int lda, scomplex *b,
	       int dispb, int ldb);
int my_dsub (int m, int n, double *a, int dispa, int lda, double *b, int dispb, int ldb,
	     double *c, int dispc, int ldc);
int my_zsub (int m, int n, complex *a, int dispa, int lda,
			complex *b, int dispb, int ldb,
	     complex *c, int dispc, int ldc);
int my_ssub (int m, int n, float *a, int dispa, int lda, float *b, int dispb, int ldb,
	     float *c, int dispc, int ldc);
int my_csub (int m, int n, scomplex *a, int dispa, int lda,
			scomplex *b, int dispb, int ldb,
	     scomplex *c, int dispc, int ldc);
int my_dgels (int trans, int m, int n, int nrhs, double *a, int dispa, int lda,
	      double *b, int dispb, int ldb, double *work, int lwork);
int my_zgels (int trans, int m, int n, int nrhs, complex *a, int dispa, int lda,
	      complex *b, int dispb, int ldb, complex *work, int lwork);
int my_sgels (int trans, int m, int n, int nrhs, float *a, int dispa, int lda,
	      float *b, int dispb, int ldb, float *work, int lwork);
int my_cgels (int trans, int m, int n, int nrhs, scomplex *a, int dispa, int lda,
	      scomplex *b, int dispb, int ldb, scomplex *work, int lwork);
int my_dtrtrs (int uplo, int trans, int diag, int n, int nrhs,
	      double a[], int dispa, int lda, double b[], int dispb, int ldb);
int my_ztrtrs (int uplo, int trans, int diag, int n, int nrhs,
	      complex a[], int dispa, int lda, complex b[], int dispb, int ldb);
int my_strtrs (int uplo, int trans, int diag, int n, int nrhs,
	       float a[], int dispa, int lda, float b[], int dispb, int ldb);
int my_ctrtrs (int uplo, int trans, int diag, int n, int nrhs,
	       scomplex a[], int dispa, int lda, scomplex b[], int dispb, int ldb);
int my_dgesvd (int jobu, int jobvt, int m, int n, double *a, int dispa, int lda,
				double *s, double *u, int dispu, int ldu, double *vt,
	       int dispvt, int ldvt, double *work, int lwork);
int my_zgesvd (int jobu, int jobvt, int m, int n, complex *a, int dispa, int lda,
				double *s, complex *u, int dispu, int ldu, complex *vt,
	       int dispvt, int ldvt, complex *work, int lwork, double *rwork);
int my_sgesvd (int jobu, int jobvt, int m, int n, float *a, int dispa, int lda,
				float *s, float *u, int dispu, int ldu, float *vt,
	       int dispvt, int ldvt, float *work, int lwork);
int my_cgesvd (int jobu, int jobvt, int m, int n, scomplex *a, int dispa, int lda,
				float *s, scomplex *u, int dispu, int ldu, scomplex *vt,
	       int dispvt, int ldvt, scomplex *work, int lwork, float *rwork);


int my_cblas_sswap (int N, float *X, int dispX, int incX, float *Y, int dispY,
		    int incY);
int my_cblas_dswap (int N, double *X, int dispX, int incX, double *Y,
		    int dispY, int incY);
int my_cblas_cswap (int N, scomplex *X, int dispX, int incX, scomplex *Y,
		    int dispY, int incY);
int my_cblas_zswap (int N, complex *X, int dispX, int incX, complex *Y,
		    int dispY, int incY);

int my_dgetrf (int m, int n, double *a, int dispa, int lda, long int *ipiv);
int my_sgetrf (int m, int n, float *a, int dispa, int lda, long int *ipiv);
int my_zgetrf (int m, int n, complex *a, int dispa, int lda, long int *ipiv);
int my_cgetrf (int m, int n, scomplex *a, int dispa, int lda, long int *ipiv);
