(*
  File: cha.ml
  Author: S. Chandrasekaran
  Date: Jan. 24, 2008
  Purpose: Computational Harmonic Analysis

  $Id$
*)

open Bigarray
open Nla.DBlop

let same_sign x1 x2 =
  let sign x = Pervasives.compare x 0.0 in
  sign x1 = sign x2

let sin_eval hf num_segments max_index x =
  let one_run si =
    let fval = hf si *. sin (float si *. x) in
    let rec ora ci acc =
      if ci > max_index then (* could not complete run *)
	(ci, 0.0)
      else begin
	let nval = hf ci *. sin (float ci *. x) in
	if (same_sign fval nval) then
	  ora (ci + 1) (acc +. nval)
	else
	  (ci, acc)
      end
    in
    ora (si+1) 0.0
  in
  let rec runs count si acc =
    if (si > max_index) or (count > num_segments) then
      acc
    else
      let (ci, tacc) = one_run si in
      Printf.printf "%i %i %g %g" count (ci-si) (abs_float tacc) acc;
      print_newline ();
      runs (count + 1) ci (acc +. tacc)
  in
  runs 0 1 0.0
(* It seems that with [hf i = 1/log i], the + and - runs remain
   approximately at the same value for long stretches, with no
   obvious monotonicity. It seems likely that plain 2^N segments
   might be better. *)

let cos_eval hf num_segments max_index x =
  let one_run si =
    let fval = hf si *. cos (float si *. x) in
    let rec ora ci acc =
      if ci > max_index then (* could not complete run *)
	(ci, 0.0)
      else begin
	let nval = hf ci *. sin (float ci *. x) in
	if (same_sign fval nval) then
	  ora (ci + 1) (acc +. nval)
	else
	  (ci, acc)
      end
    in
    ora (si+1) 0.0
  in
  let rec runs count si acc =
    if (si > max_index) or (count > num_segments) then
      acc
    else
      let (ci, tacc) = one_run si in
      Printf.printf "%i %i %g %g" count (ci-si) (abs_float tacc) acc;
      print_newline ();
      runs (count + 1) ci (acc +. tacc)
  in
  runs 0 1 0.0

let sin_eval_exp hf num_segments x =
  let rec one_run ci mi acc =
    if ci > mi then
      (ci, acc)
    else
      one_run (ci+1) mi (acc +. hf ci *. sin (float ci *. x))
  in
  let rec run cr ci acc =
    if cr > num_segments then
      acc
    else
      let (ni, tacc) = one_run ci (ci + (1 lsl cr)) 0.0 in
      Printf.printf "%i %i %g %g" cr ci tacc acc;
      print_newline ();
      run (cr + 1) ni (acc +. tacc)
  in
  run 0 1 0.0


let cos_eval_exp hf num_segments x =
  let rec one_run ci mi acc =
    if ci > mi then
      (ci, acc)
    else
      one_run (ci+1) mi (acc +. hf ci *. cos (float ci *. x))
  in
  let rec run cr ci acc =
    if cr > num_segments then
      acc
    else
      let (ni, tacc) = one_run ci (ci + (1 lsl cr)) 0.0 in
      Printf.printf "%i %i %g %g" cr ci tacc acc;
      print_newline ();
      run (cr + 1) ni (acc +. tacc)
  in
  run 0 1 0.0
